#!/bin/sh

#
# diff2html.sh - affiche les differences entre deux version d'un document
#                en HTML.
#
# Usage : diff2html.sh [ -n ] [ -2 ] <fic1> <fic2>
#
# Description :
# diff2html.sh presente le fichier <fic1> avec les
# modification a apporter pour obtenir le fichier <fic2>. Les lignes
# a rajouter, enlever et/ou modifier sont affichees en couleur.
#
# Options :
# <fic1> designe la premiere version du fichier
# <fic2> designe la seconde version du fichier
# [-n] permet d'afficher le numero de chaque ligne
# [-2] permet d'avoir les deux versions du fichiers dans deux colones
# separees, pour pouvoir comparer.
#
# Couleurs :
# Par defaut, les couleurs des lignes a modifier sont les suivantes :
# 	a rajouter : rouge
# 	a modifier : bleu
# 	a supprimer : rouge barre
# Ces couleurs sont modifiables directement dans le script, en modifiant
# les variables <CouleurA>, <CouleurC> et <CouleurD>.
#
# Voir aussi :
# test-diff2html.sh(1)
#

#
# Les couleurs utilisees pour l'affichage.
#

CouleurA="red"
CouleurC="blue"
CouleurD="red"
Tab="    " # Une tabulation = 4 espaces

#
# NE RIEN MODIFIER AU DELA DE CETTE LIGNE
#

[ $# -lt 2 ] && echo >&2 "Usage : `basename $0` [ -n ] <fic1> <fic2>" && exit 1

while [ -n "$1" ]
do
    case "$1" in
        -n) Nb="y" ;;
        -2) Tablo="y" ;;
        *) Fic1=$1 ; Fic2=$2 ; shift ;;
    esac
    shift
done

if [ ! -f $Fic1 ] || [ ! -f $Fic2 ]
then
    ls $Fic1 $Fic2
    exit 2
fi

INDENT="./indent.sh"

#
# La commande diff permet de connaitre les differences entre les deux fichiers.
#

echo "$Fic1" | grep "^/" > /dev/null || Fic1="`pwd`/$Fic1"
echo "$Fic2" | grep "^/" > /dev/null || Fic2="`pwd`/$Fic2"
cd `dirname $0`

diff $Fic1 $Fic2 > /tmp/diff.$$

cat << EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head><title>diff2html.sh entre $Fic1 et $Fic2</title></head>

<body bgcolor="#ffffff">
<div align="center"><h1><tt>diff2html.sh</tt> entre $Fic1 et $Fic2</h1></div>
<p><pre>
<b>$LOGNAME@`hostname` $</b> ls -li $Fic1 $Fic2
`ls -li $Fic1`
`ls -li $Fic2`
<b>$LOGNAME@`hostname` $</b>
</pre>

<p>Rappel des couleurs utilises :

<table>
<tr><td>Ajout</td><td><font color="$CouleurA">$CouleurA</font></td></tr>
<tr><td>Changement</td><td><font color="$CouleurC">$CouleurC</font></td></tr>
<tr><td>Effacement</td><td><font color="$CouleurD"><strike>$CouleurD</strike></font></td></tr>
</table>
<hr>

<p>
EOF

[ "$Tablo" = "y" ] && cat << EOF
<table width="100%" border="0">
<tr><td width="50%" align="center">$Fic1</td><td width="50%" align="center">$Fic2</td></tr>
EOF

MaxLignes=`wc -l $Fic1 | sed "s/^ *\([0-9][0-9]*\) .*$/\1/"`

NoLigne=0

NombreChangements=0

Image ()
{
    RepImages=`pwd`
    set -- `ls -l $0`
    shift 9
    if [ "$1" = "->" ]
    then
        RepImages=`dirname $2`
        echo "$RepImages" | grep "^/" > /dev/null || RepImages="`pwd`/$RepImages"
    fi
   
    if [ $NombreChangements -eq 0 ]
    then
        ChangementPrecedent=0
        ChangementSuivant=1
    else
        ChangementPrecedent=`expr $NombreChangements - 1`
        ChangementSuivant=`expr $NombreChangements + 1` 
    fi
    cat << EOF
<a href="#$ChangementPrecedent"><img src="$RepImages/up.gif" height="20" border="0"></a>
<a href="#$ChangementSuivant"><img src="$RepImages/down.gif" height="20" border="0"></a>
<a name="$NombreChangements"></a>
EOF
    NombreChangements=`expr $NombreChangements + 1`
}

#
# Formattage pour le HTML :
#     - le caractere "<" est interprete, il faut le transformer en "&lt;" ;
#     - une tabulation est affichee comme un espace, il faut le transformer
# en 4 espaces (par defaut) ;
#     - un espace doit etre transforme en "&nbsp;" ;
#     - le caractere "\" doit etre transforme en "\\" pour que read ne
# l'interprete pas.
#

cat $Fic1 |
sed -e "s/</\&lt;/g" \
    -e "s/	/$Tab/g" \
    -e "s/ /\&nbsp;/g" \
    -e "s/\\\\/\\\\\\\\/g" | \
while read ligne
do
    NoLigne=`expr $NoLigne + 1`
    Pct=`expr $NoLigne \* 100 / $MaxLignes`
    echo -en >&2 "Performing line $NoLigne...                                            $Pct % done\r"

#
# FIXME
# Remplacer le code :
#  awk '{ if ($0 ~ /^<tr>/) { gsub ("<tr>", "") ; print "<tr><td>" NR "</td>" $0 } else { print $0 } }' /tmp/diff2.html > /tmp/diff3.html
# par :
#

    [ -n "$Nb" ] && echo -n "$NoLigne&nbsp;"

#
# Pour la ligne courante du premier fichier, des modifications doivent-elles
# etre apportees ?
#

    Diff=`grep "^$NoLigne[acd,]" /tmp/diff.$$`

#
# Append : rajouter a partir de la ligne l1 du premier fichier la (les)
# ligne(s) l2 (l2 a l3) dans le second.
#
# Exemples :
#   l1al2
#   l1al2,l3
#

    if echo "$Diff" | grep "^${NoLigne}a.*" > /dev/null
    then
#echo "$NoLigne> $Diff => Append" >> /tmp/diff2html.log

        if [ "$Tablo" = "y" ]
        then
            echo -n "<tr><td width=\"50%\">"
            Image
            echo -n "&nbsp;</td><td width=\"50%\">"
        else
            Image
        fi
        echo "$ligne<br>"
        #cat /tmp/diff.$$ | $INDENT | \
        cat /tmp/diff.$$ | \
            awk '\
                BEGIN { p=0 } \
                /'"$Diff"'/ { p=1 ; next } \
                /^[0-9]/ { p=0 } \
                p ~ 1 { sub ("^> ", "") ; \
                        gsub ("<", "\\&lt;") ; \
                        gsub ("	", "'"$Tab"'") ; \
                        gsub (" ", "\\&nbsp;") ; \
                        print "<font color=\"'"$CouleurA"'\">" $0 "</font><br>" } \
                ' #< /tmp/diff.$$

        if [ "$Tablo" = "y" ]
        then
            echo "</td></tr>"
        fi

#
# Delete : effacer la (les) ligne(s) l1 (l1 a l2) du premier fichier
# a partir de la ligne l3 dans le second.
#
#
# Exemples :
#   l1dl3
#   l1,l2dl3
#

    elif echo "$Diff" | egrep "^${NoLigne}d[0-9]+" > /dev/null
    then
#echo "$NoLigne> $Diff => Delete" >> /tmp/diff2html.log
        if [ "$Tablo" = "y" ]
        then
            echo "<tr><td width=\"50%\">"
            Image
            echo "$ligne</td><td width=\"50%\"><font color=\"$CouleurD\"><strike>$ligne</strike></font></td></tr>"
        else
            echo "<font color=\"$CouleurD\">"
            Image
            echo "<strike>$ligne</strike></font><br>"
        fi
    elif echo "$Diff" | egrep "^${NoLigne},[0-9]+d[0-9]+" > /dev/null
    then
        if [ "$Tablo" = "y" ]
        then
            echo "<tr><td width=\"50%\">"
            Image
            echo "$ligne</td><td width=\"50%\"><font color=\"$CouleurD\"><strike>$ligne</strike></font></td></tr>"
        else
            echo "<font color=\"$CouleurD\">"
            Image
            echo "<strike>$ligne</strike></font><br>"
        fi
        DerniereLigne="`echo $Diff | sed -e s/d.*// -e s/.*,//`"
        while read ligne2
        do
            if [ "$Tablo" = "y" ]
            then
                echo -n "<tr><td width=\"50%\">"
                Image
                echo "$ligne2</td><td width="50%">"
                echo "$ligne2" | \
                sed -e "s/^/<font color=\"$CouleurD\"><strike>/" \
                    -e "s/$/<\/strike><\/font><br>/"
                echo "</td></tr>"
            else
                echo "$ligne2" | \
                sed -e "s/^/<font color=\"$CouleurD\"><strike>/" \
                    -e "s/$/<\/strike><\/font><br>/"
            fi
            NoLigne=`expr $NoLigne + 1`
            [ $NoLigne -eq $DerniereLigne ] && break
        done

#
# Change : remplacer la (les) ligne(s) l1 (l1 a l2) dans le premier fichier
# par la (les) ligne(s) l3 (l3 a l4) dans le second.
#
# Exemples :
#   l1cl3
#   l1,l2cl3
#   l1cl3,l4
#   l1,l2cl3,l4
#

    elif echo "$Diff" | egrep "^${NoLigne}.*c" > /dev/null
    then
#echo "$NoLigne> $Diff => Change " >> /tmp/diff2html.log
        LigneCI=`echo $Diff | cut -dc -f1`
        LigneCO=`echo $Diff | cut -dc -f2`

        if [ "$Tablo" = "y" ]
        then
            echo -n "<tr><td width=\"50%\">"
            Image
#$INDENT | \
            sed -n ${LigneCI}p < $Fic1 | \
            sed -e "s/</\&lt;/g" \
                -e "s/	/$Tab/g" \
                -e "s/ /\&nbsp;/g" \
                -e "s/^/<font color=\"$CouleurC\">/" \
                -e "s/$/<\/font><br>/"
            echo -n "</td><td width=\"50%\">"
#$INDENT | \
            sed -n ${LigneCO}p < $Fic2 | \
            sed -e "s/</\&lt;/g" \
                -e "s/	/$Tab/g" \
                -e "s/ /\&nbsp;/g" \
                -e "s/^/<font color=\"$CouleurC\">/" \
                -e "s/$/<\/font><br>/"
                echo "</td></tr>"
        else
            sed -n ${LigneCO}p < $Fic2 | \
            sed -e "s/</\&lt;/g" \
                -e "s/	/$Tab/g" \
                -e "s/ /\&nbsp;/g" \
                -e "s/^/<font color=\"$CouleurC\">/" \
                -e "s/$/<\/font><br>/"
        fi

        if echo "$LigneCI" | grep "," > /dev/null
        then
            NouvelleLigne=`echo $LigneCI | cut -d, -f2`
            while [ $NoLigne -ne $NouvelleLigne ]
            do
                read
                NoLigne=`expr $NoLigne + 1`
            done
        fi

#
# Ligne sans modifications a apporter : elle est copiee telle quelle.
#

    else
#echo "$NoLigne> $Diff => aucune modif" >> /tmp/diff2html.log
        if [ "$Tablo" = "y" ]
        then
            echo "<tr><td width=\"50%\">"
#echo "$ligne" | sed "s/\&nbsp;/ /g" | $INDENT | sed "s/$/<br>/"
#echo "$ligne" | sed "s/\&nbsp;/ /g" | sed "s/$/<br>/"
            echo "$ligne<br>"
            echo "</td><td width="50%">"
#echo "$ligne" | sed "s/\&nbsp;/ /g" | $INDENT | sed "s/$/<br>/"
echo "$ligne" | sed "s/\&nbsp;/ /g" | sed "s/$/<br>/"
            echo "</td></tr>"
        else
            echo "$ligne<br>"
        fi
    fi
done

[ "$Tablo" = "y" ] && cat << EOF
</table>
EOF

rm -f /tmp/diff.$$

cat << EOF
</body>
</html>
EOF

echo >&2

