#!/bin/sh

#
# test-diff2html.sh - test le programme diff2html.sh.
#
# Usage : test-diff2html.sh <path1> <path2> [pattern]
#
# Description :
# test-diff2html.sh permet de tester le programme
# <diff2html.sh> en comparant tous les fichiers de deux version d'un
# projet.
# Pour effectuer le test, tous les fichiers C du noyau de linux sont
# comparer entre deux versions (chez moi, entre la 2.2.16 et la 2.4.17).
# Mais le test pourrait se faire sur n'importe quel projet, le noyau
# Linux a simplement ete choisit en raison de sa taille et de son
# dynamisme (et donc des modifications nombreuses).
#
# Options :
# <path1> est le chemin de la premiere version du projet
# <path2> est le chemin de la seconde version du projet
# <pattern> est la chaine de caracteres qui servira de modele pour la
# recherche des noms des fichiers. Par defaut, tous les fichiers se
# terminant par "*.c" seront seront recherches.
#
# Exemples d'utilisation :
# ./test-diff2html.sh linux 2.4.17/linux
# cherche les differences entre tous les fichiers C des repertoires
# linux et 2.4.17/linux,
# ./test-diff2html.sh linux 2.4.17/linux/ *.h
# cherche les differences entre tous les fichiers dont le nom se termine
# par "*.h" des repertoires linux et 2.4.17/linux.
#
# Voir aussi :
# diff2html.sh(1)
#

[ $# -ne 2 -a $# -ne 3 ] && echo >&2 "Usage : test-diff2html.sh <path1> <path2>" && exit 1

#
# Quelle extension les fichiers a comparer doivent-ils avoir ?
#

[ -n "$3" ] && Pattern="$3" || Pattern="*.c"

cd $1
for fic in `find . -name "$Pattern"`
do
    echo -n "`pwd`/$fic vs. "
    if [ -f $2/$fic ]
    then
        echo "$2/$fic :"
        diff $fic $2/$fic > /tmp/diff.$$
        grep -v "^[<>-]" < /tmp/diff.$$
        ls -s /tmp/diff.$$
        cat /tmp/diff.$$
        rm -f /tmp/diff.$$
    else
        echo "nil."
    fi
    echo
done

